
// COLORS
color vlogC = #47C0B9; // 1
color cookingC = #35C0CC; // 2
color gamerC = #29AED1; // 3
color musicC = #3299CC; // 4
color danceC= #3781C4; // 5
color entC= #4071B7; // 6
color otherC = #455CAA; // 7
color titleC; // Default color
color darkGray = #58595B; // Dark Gray Color
color lightGray = #808285; // Light gray color
color lightGray2 = #E0E0E0;

// XML & YOUTUBERS
XML root;
XML[] channels;
XML cTitleElement;
XML cNameElement;
XML cCategoryElement;
XML[] cCList;
XML[] allSubCountsElement;
int[] allSubCounts;
XML cSubsElement;
int cSubs;
String[] cConnect;
String cTitle;
String cName;
String cCategory;
Youtuber[] channelObjects;
ArrayList<Connection> connectionLines;
XML[] connectTitlesElement; // List of all titles for the purpose of connections
String[] connectTitles; // Compiled into strings
int tierLevel; // Finds the tier level of each connection
StringList cConnectString; // list of all the connection strings
XML cMostWatchedElement;
String cMostWatched;
String cMostWatchedViews;
String cMostWatchedURL;

// YOUTUBER IMAGES
XML cImageElement;
String cImage;

// COUNTERS
Counter youtubersCounter;

// Instructions
Instructions instructions;

// SUB BARS
SubBars lowSubBar; // low sub count bar
SubBars averageSubBar; // average sub count bar
SubBars channelSubBar; // channel's sub count bar
SubBars highSubBar; // high sub count bar
float subs;
float subLow; // Lowest sub count
float subHigh; // Highest sub count
float subAverage; // Average sub count

// FONTS
PFont titleFont; // Title Line
PFont nameFont; // Name Line && Counter Numbers
PFont av12Font; // Size 12 font
PFont av12BoldFont; // Size 12 font bold
PFont av18Font; // Size 18 font
PFont tooltip; // hover tooltip text
PFont av8Font;

// Beginning test
String startText1 = "WHAT DOES 2013 SAY?";
String startText2 = "Youtube Rewind";
float startTitleTextY;
float startTitleTextY2;
float titleTextY = 720; // final text position
float titleTextY2 = height+100; // Offscreen position
boolean moveTitle = false;

// OTHER VARIABLES
float radius;
int numChannels;
float angle;
float transX;
float transY;
boolean anyClicked = false; // Check to see if any of the channels are clicked
boolean anyHover = false; // Check to see if any of the channels is hovered on

void setup(){
  size(1400,800);
  frameRate(30);
  noStroke();
  
  //FONTS
  titleFont = loadFont("Aachen-Bold-50.vlw");
  nameFont = loadFont("Serifa-Bold-45.vlw");
  av12Font = loadFont("AvenirNext-Regular-12.vlw");
  av12BoldFont = loadFont("AvenirNext-Bold-12.vlw");
  av18Font = loadFont("AvenirNext-Regular-18.vlw");
  av8Font = loadFont("AvenirNext-Regular-8.vlw");
  tooltip = loadFont("AvenirNext-Regular-14.vlw");
  
  
  // SET UP YOUTUBERS
  root = loadXML("youtubers.xml"); // Load XML File
  channels = root.getChildren("channel"); // Put all channels into array
  channelObjects = new Youtuber[channels.length]; // Make an array of youtubers with how many are in the xml
  connectionLines = new ArrayList<Connection>(); // Make an expandable array for the connection lines
  
  allSubCountsElement = root.getChildren("channel/subs"); // Get all subscriber counts for high, low and average
  allSubCounts = new int[allSubCountsElement.length];
  for (int k = 0; k < allSubCountsElement.length; k++){
    allSubCounts[k] = allSubCountsElement[k].getIntContent();
  }
  
  
  // OTHER VARIABLES
  transX = width/2;
  transY = 350;
  radius = 280; // circle radius
  angle=TWO_PI/(float)channels.length; // angle each circle needs to go
  // for sub bars
  subLow = min(allSubCounts); 
  subHigh = max(allSubCounts);
  float average = 0; // Compute the average
   for ( int i = 0; i < allSubCounts.length; i++ ) { 
     subAverage += allSubCounts[i]; 
   } 
   subAverage /= (float)(allSubCounts.length); 
   subs = 0; // temp number
   cConnectString = new StringList();
   startTitleTextY = 720;
   startTitleTextY2 = height+100;
   
  
  // OTHER OBJECTS
  youtubersCounter = new Counter("youtubers");
  instructions = new Instructions();
  lowSubBar = new SubBars(0, subLow, "Lowest");
  averageSubBar = new SubBars(1, subAverage, "Average");
  channelSubBar = new SubBars(2, subs, "Channel");
  highSubBar = new SubBars(3, subHigh, "Highest");

     // SET UP CONNECTIONS
   connectTitlesElement = root.getChildren("channel/title");
   connectTitles = new String[connectTitlesElement.length];
   for (int n=0; n < connectTitlesElement.length; n++){
     connectTitles[n] = connectTitlesElement[n].getContent();
   }

  // SET UP YOUTUBERS PART 2
  for (int i = 0; i < channels.length; i++) {
    cTitleElement = channels[i].getChild("title");
    cTitle = cTitleElement.getContent();
    cNameElement = channels[i].getChild("name");
    cName = cNameElement.getContent();
    cCategoryElement = channels[i].getChild("category");
    cCategory = cCategoryElement.getContent();
    cImageElement = channels[i].getChild("img");
    cImage = cImageElement.getContent();
    cCList = channels[i].getChildren("connectList/connect"); // Gets connectList
    cConnect = new String[cCList.length];
    cSubsElement = channels[i].getChild("subs"); // gets subscriber count
    cSubs = cSubsElement.getIntContent();
    cMostWatchedElement = channels[i].getChild("watched"); // most watched
    cMostWatched = cMostWatchedElement.getContent();
    cMostWatchedViews = cMostWatchedElement.getString("views");
    cMostWatchedURL = cMostWatchedElement.getString("url", "blank");
    
    for (int k = 0; k < cCList.length; k++){
      cConnect[k] = cCList[k].getContent(); // Found the connections and converted them to strings
      tierLevel = cCList[k].getInt("tier"); // Gets the tier level of the connection
      
      for (int m=0; m < connectTitles.length; m++) { // Run through each channel title for a match
        if (cConnect[k].equals(connectTitles[m])){
          connectionLines.add(new Connection(i, m, tierLevel, cCategory)); // add a new connection line 
        } else { }
      } 
    }
    channelObjects[i] = new Youtuber(radius*sin(angle*i), -(radius*cos(angle*i)), cTitle, cCategory, cName, cImage, cSubs, cConnect, cMostWatched, cMostWatchedViews, cMostWatchedURL); 
  }
   
   // DEFAULT TEXT
  titleC = gamerC; // Default color
   
   // Adjust connection lines
  for (int i = 0; i < channels.length; i++) {
    channelObjects[i].xpos2 = channelObjects[i].xpos;
    channelObjects[i].ypos2 = channelObjects[i].ypos;
  }

}

void draw(){
  background(255);
  pushMatrix();
  translate(transX, transY); // center of circle
  anyHover = false;
  
  // DISPLAY CONNECTIONS
  for (int i=0; i < connectionLines.size(); i++){
    Connection connection = connectionLines.get(i);
    connection.display();
  }
    
  // DISPLAY YOUTUBERS
  for (int i = 0; i < channels.length; i++) {
    channelObjects[i].display();
  }
  // Go back and display any showing tooltips again to keep them above the circles
  for (int i = 0; i < channels.length; i++){
    channelObjects[i].showTip();
  }
  
  for (int i = 0; i < channels.length; i++) {
   // Check to see if any channels are clicked
    if (channelObjects[i].ifClicked == true) {
      anyClicked = true;
      cConnectString.clear(); // erase all current connections
      // If a channel is clicked, hide all connections that aren't with that channel
      for (int k = 0; k < connectionLines.size(); k++){
        Connection connection = connectionLines.get(k);
        if (connection.person1 != i) {
          if (connection.person2 != i) {
             connection.showConnection = false;
          }
        } 
         // if a connection is showing, add it to the cConnectString stringlist 
         if (connection.showConnection == true) {
           if (connection.person1 != i) {
             cConnectString.append(channelObjects[connection.person1].title);
           } else {
             cConnectString.append(channelObjects[connection.person2].title);
           }
         }
      }
    } else { }
   // Check to see if any channels has hover on
   if (channelObjects[i].ifHovered == true) {
      anyHover = true;
    } else {}
  }
  // if no channels are clicked, have cConnectString show full youtuber list instead
  if (anyClicked == false) {
    cConnectString.clear(); // remove any strings in the list
    for (int i = 0; i < channels.length; i++) {
      cConnectString.append(channelObjects[i].title);
    }
  }
  
  
  // adjust channel sub value
  for (int i = 0; i < channels.length; i++){
    if (channelObjects[i].ifClicked == true){
      channelSubBar.subCounter = channelObjects[i].cSubs;
      channelSubBar.c = channelObjects[i].c;
    }
  }
  // draw sub bars but don't display until channel is selected
  highSubBar.display();
  channelSubBar.display();
  averageSubBar.display();
  lowSubBar.display();
  //draw subbar title text once channel is selected
  if(anyClicked == true){
    textFont(av18Font);
    for (int i = 0; i < channels.length; i++){
      if (channelObjects[i].ifClicked == true) {
        fill(channelObjects[i].c);
      }
    }
    textAlign(RIGHT);
    text("Subscriber Counts", (highSubBar.x-10), (height-transY-20));
  }
  
  popMatrix();
  
  // DISPLAY TEXT
  textFont(titleFont);
  textAlign(CENTER);
  fill(titleC);
  rectMode(CORNER);
  text(startText1.toUpperCase().trim(), (width/2), startTitleTextY); // Title Text
  textFont(nameFont);
  fill(darkGray);
  text((startText2.toLowerCase().trim()), (width/2), startTitleTextY+35); // Name Text

  // animate text out on click
  if (anyClicked == true){
    float distY = startTitleTextY2 + startTitleTextY;
    if (startTitleTextY < startTitleTextY2) {
      startTitleTextY = startTitleTextY + distY/8;
    }
  }  else {
    // animate text back in off click
    float distY = 720 - startTitleTextY;
    if (startTitleTextY > 720) {
      startTitleTextY = startTitleTextY + distY/4;
    }
  }
  
  // DISPLAY YOUTUBER COUNT
  youtubersCounter.display();
  
  // display instructions
  instructions.display();
  
  
}
